/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctExportOptionWidget.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctExportOptionWidget.h"
#include "ctSignalRouter.h"
#include "ctGlobal.h"

#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>

/*------------------------------------------------------------ Functions */

/** Constructor for the \a ctExportOptionWidget class.
@param parent Pointer to the parent widget
@param name Internal name for the new \a ctExportOptionWidget
*/
ctExportOptionWidget::ctExportOptionWidget( QWidget *parent, const char *name )
    : QVBox( parent, name )
{

  // Get group box for LaTeX options 
  QGroupBox *gbLatex = new QGroupBox(4, QGroupBox::Vertical, tr("LaTeX"), this);

  // Get combo box for paper size
  QHBox *hbPaperSize = new QHBox( gbLatex );
  cbPaperSize = new QComboBox(hbPaperSize, "cbPaperSize");
  cbPaperSize->insertItem(tr("ISO A4"));
  cbPaperSize->insertItem(tr("US letter"));
#ifdef DEFENGLISH
  cbPaperSize->setCurrentItem(1);
#endif
  QLabel *lLabel = new QLabel(tr("Paper size"), hbPaperSize);

  // Get check box for twocolumn option
  cbTwoColumns = new QCheckBox(tr("Output in two columns"), gbLatex, "cbTwoColumns");
  cbTwoColumns->setChecked(true);

  // Get combo box for font size of diagrams
  QHBox *hbDiagramSize = new QHBox( gbLatex );
  cbLaTeXDiagramSize = new QComboBox(hbDiagramSize, "cbLaTeXDiagramSize");
  cbLaTeXDiagramSize->insertItem(tr("Tiny"));
  cbLaTeXDiagramSize->insertItem(tr("Small"));
  cbLaTeXDiagramSize->insertItem(tr("Normal"));
  cbLaTeXDiagramSize->insertItem(tr("Large"));
  lLabel = new QLabel(tr("Diagrams"), hbDiagramSize);
  // Get combo box for font size
  QHBox *hbFontSize = new QHBox( gbLatex );
  cbFontSize = new QComboBox(hbFontSize, "cbFontSize");
  cbFontSize->insertItem("10pt");
  cbFontSize->insertItem("11pt");
  cbFontSize->insertItem("12pt");
  lLabel = new QLabel(tr("Font size"), hbFontSize);

  // Get group box for HTML options 
  QGroupBox *gbHTML = new QGroupBox(4, QGroupBox::Vertical, tr("HTML"), this);
  // Get check box for exporting to one file only
  cbOneFile = new QCheckBox(tr("Export to single file"), gbHTML, "cbOneFile");
  cbOneFile->setChecked(false);
  // Get check box for expanding informator symbols to plain text
  cbUseTTFont = new QCheckBox(tr("Use TrueType font"), gbHTML, "cbUseTTFont");
  cbUseTTFont->setChecked(true);
  // Get check box for expanding informator symbols to plain text
  cbExpandSymbols = new QCheckBox(tr("Informator symbols in plain text"), gbHTML, "cbExpandSymbols");
  cbExpandSymbols->setChecked(false);
  cbExpandSymbols->setEnabled(false);
  // Get spin box for the number of directory levels to the graphics directory
  QHBox *hbDirLevel = new QHBox( gbHTML );
  sbDirLevel = new QSpinBox(1, 20, 1 , hbDirLevel, "sbDirLevel");
  sbDirLevel->setValue(3);
  lLabel = new QLabel(tr("Levels to graphics directory"), hbDirLevel);

  // Get group box for General options 
  QGroupBox *gbGeneral = new QGroupBox(1, QGroupBox::Horizontal, tr("General"), this);
  // Get check box for export with/without solutions
  cbOutputSolutions = new QCheckBox(tr("Export with solutions"), gbGeneral, "cbOutputSolutions");
  cbOutputSolutions->setChecked(true);

  setupConnects();

}


/** This function sets up the connects for this widget. */
void ctExportOptionWidget::setupConnects()
{

  // Setup connects for the checkboxes and the spinbox
  connect(cbPaperSize, SIGNAL( activated(int) ),
          this, SLOT( slotPaperSizeChanged(int) ));
  connect(cbLaTeXDiagramSize, SIGNAL( activated(int) ),
          this, SLOT( slotLaTeXDiagramSizeChanged(int) ));
  connect(cbTwoColumns, SIGNAL( clicked() ),
          this, SLOT( slotTwoColumnsEnableChanged() ));
  connect(cbOneFile, SIGNAL( clicked() ),
          this, SLOT( slotOneFileEnableChanged() ));
  connect(cbUseTTFont, SIGNAL( clicked() ),
          this, SLOT( slotUseTTFontEnableChanged() ));
  connect(cbExpandSymbols, SIGNAL( clicked() ),
          this, SLOT( slotExpandSymbolsEnableChanged() ));
  connect(cbOutputSolutions, SIGNAL( clicked() ),
          this, SLOT( slotOutputSolutionsEnableChanged() ));
  
  // Setup connections to the signal router
  connect(this, SIGNAL( sigSetA4PaperSizeEnable(bool) ),
          srChessTask, SLOT( slotSetA4PaperSizeEnable(bool) ));
  connect(this, SIGNAL( sigSetLaTeXDiagramSize(int) ),
          srChessTask, SLOT( slotSetLaTeXDiagramSize(int) ));
  connect(this, SIGNAL( sigSetTwoColumnsEnable(bool) ),
          srChessTask, SLOT( slotSetTwoColumnsEnable(bool) ));
  connect(this, SIGNAL( sigSetOneFileEnable(bool) ),
          srChessTask, SLOT( slotSetOneFileEnable(bool) ));
  connect(this, SIGNAL( sigSetUseTTFontEnable(bool) ),
          srChessTask, SLOT( slotSetUseTTFontEnable(bool) ));
  connect(this, SIGNAL( sigSetExpandSymbolsEnable(bool) ),
          srChessTask, SLOT( slotSetExpandSymbolsEnable(bool) ));
  connect(this, SIGNAL( sigSetOutputSolutionsEnable(bool) ),
          srChessTask, SLOT( slotSetOutputSolutionsEnable(bool) ));
  connect(sbDirLevel, SIGNAL( valueChanged(int) ),
          srChessTask, SLOT( slotSetDirLevel(int) ));
  connect(cbFontSize, SIGNAL( activated(int) ),
          srChessTask, SLOT( slotSetFontSize(int) ));

}


/*---------------------------------------------------------------- Slots */


/** Emits the \a sigSetA4PaperSize signal
and sets the paper size for the LaTeX output. 
@param index Index of the activated combobox item
*/
void ctExportOptionWidget::slotPaperSizeChanged(int index)
{
  if (index == 0)
    emit sigSetA4PaperSizeEnable(true);  // ISO A4 paper size
  else
    emit sigSetA4PaperSizeEnable(false); // US letter paper size
}

/** Emits the \a sigSetLaTeXDiagramSize signal
and sets the font size for the LaTeX diagrams. 
@param index Index of the activated combobox item
*/
void ctExportOptionWidget::slotLaTeXDiagramSizeChanged(int index)
{

  // Twocolumn output with large font?
  if ((cbTwoColumns->isChecked()) && (index == LARGE))
  {
    // No way...
    cbTwoColumns->setChecked(false);
    emit sigSetTwoColumnsEnable(false);
  }

  emit sigSetLaTeXDiagramSize(index);
}

/** Emits the \a sigSetTwoColumnsEnable signal
to enable or disable the output in two columns. 
*/
void ctExportOptionWidget::slotTwoColumnsEnableChanged()
{
  // Twocolumn output with large font?
  if ((cbTwoColumns->isChecked()) && (cbLaTeXDiagramSize->currentItem() == LARGE))
  {
    // No way...
    cbLaTeXDiagramSize->setCurrentItem(NORMAL);
    emit sigSetLaTeXDiagramSize(NORMAL);
  }

  emit sigSetTwoColumnsEnable(cbTwoColumns->isChecked());
}

/** Emits the \a sigSetOneFileEnable signal
to enable or disable the HTML output into a single file. 
*/
void ctExportOptionWidget::slotOneFileEnableChanged()
{
  emit sigSetOneFileEnable(cbOneFile->isChecked());
}

/** If the usage of the TrueType font ``DBChess'' was
enabled, the CheckBox ``Export symbols as plain text'' gets
inactive. Else, the CheckBox for the
informator symbols is enabled again and can be set to 
\c true or \c false by the user.
*/
void ctExportOptionWidget::slotUseTTFontEnableChanged()
{
  if (cbUseTTFont->isChecked())
  {
    // Disable ``informator symbols in plain text''
    cbExpandSymbols->setEnabled(false);
    // Set option ``Use TrueType font'' to ``enabled''
    emit sigSetUseTTFontEnable(true);
  }
  else
  {
    // Enable CheckBox ``informator symbols in plain text'' again
    cbExpandSymbols->setEnabled(true);
    // Set option ``Use TrueType font'' to ``false''
    emit sigSetUseTTFontEnable(false);
  }
}

/** Emits the \a sigSetExpandSymbolsEnable signal
to enable or disable the output of the informator symbols in plain text. 
*/
void ctExportOptionWidget::slotExpandSymbolsEnableChanged()
{
  emit sigSetExpandSymbolsEnable(cbExpandSymbols->isChecked());
}

/** Emits the \a sigSetNoSolutionsEnable signal
to enable or disable the output of solutions for the export. 
*/
void ctExportOptionWidget::slotOutputSolutionsEnableChanged()
{
  emit sigSetOutputSolutionsEnable(cbOutputSolutions->isChecked());
}

