/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctOption.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctOption.h"
#include "ctPixmaps.h"
#include "ctPieces.h"
#include "ctGlobal.h"
#include "ctTitleDialog.h"
#include "ctDrawBoard.h"
#include "ctTaskList.h"
#include "ctPosition.h"
#include "ctProgressDialog.h"
#include "ctSignalRouter.h"
#include "ctHTMLExport.h"

#include <qapplication.h>
#include <qwidget.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#ifdef QTVER2X
#include <qmultilineedit.h>
#else
#include <qtextedit.h>
#endif
#include <qcolor.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qaccel.h>

/*------------------------------------------------------------ Functions */

/** Constructor for the \a ctOption class.
@param parent Pointer to the parent widget
@param name Internal name for the new \a ctOption widget
*/
ctOption::ctOption( QWidget *parent, const char *name )
    : QVBox( parent, name),
      currentColor(WHITE),
      currentPiece(PAWN),
      graphicsDir("../../../graphics/")
{
  // Get progress window
  pdProgress = new ctProgressDialog(0, "pdProgress");
  pdProgress->hide();
  // Get list of tasks
  tlTasks = new ctTaskList;
  
  setupOptions();
  setupConnects();
}


/** Destructor for the \a ctOption class.
*/
ctOption::~ctOption()
{
  // Delete the list of tasks
  delete tlTasks;
}  

/** Sets up the options for the board like inserting a task, deleting,
moving, setting color and piece a.s.o. */
void ctOption::setupOptions()
{
  // Get new group box
  QGroupBox *gbTaskTitle = new QGroupBox(1, QGroupBox::Horizontal, tr("Title of the task"), this);
  // Get line edit for the title
  leTitle = new QLineEdit(gbTaskTitle, "leTitle");

  // Get horizontal box for drawing area and the buttons
  QHBox *hbDrawAndButtons = new QHBox( this );

  // Get drawing board
  dbDrawingBoard = new ctDrawBoard( hbDrawAndButtons, "dbDrawingBoard");

  // Get horizontal box for all buttons
  QHBox *hbButtons = new QHBox( hbDrawAndButtons );

  // Get vertical box for left side with pieces a.s.o
  QVBox *vbPieces = new QVBox( hbButtons );

  // Get new group box
  QGroupBox *gbPieces = new QGroupBox(4, QGroupBox::Horizontal, tr("Pieces"), vbPieces);
  // Get button for pawn 
  pbSetPawn = new QPushButton(gbPieces, "pbSetPawn");
  pbSetPawn->setPixmap(pixMaps[PAWN]);
  // Get button for knight 
  pbSetKnight = new QPushButton(gbPieces, "pbSetKnight");
  pbSetKnight->setPixmap(pixMaps[KNIGHT]);
  // Get button for bishop 
  pbSetBishop = new QPushButton(gbPieces, "pbSetBishop");
  pbSetBishop->setPixmap(pixMaps[BISHOP]);
  // Get button for marker A 
  pbSetMarkerA = new QPushButton(gbPieces, "pbSetMarkerA");
  pbSetMarkerA->setPixmap(pixMaps[MARKER_A]);
  // Get button for rook 
  pbSetRook = new QPushButton(gbPieces, "pbSetRook");
  pbSetRook->setPixmap(pixMaps[ROOK]);
  // Get button for queen 
  pbSetQueen = new QPushButton(gbPieces, "pbSetQueen");
  pbSetQueen->setPixmap(pixMaps[QUEEN]);
  // Get button for king 
  pbSetKing = new QPushButton(gbPieces, "pbSetKing");
  pbSetKing->setPixmap(pixMaps[KING]);
  // Get button for marker B 
  pbSetMarkerB = new QPushButton(gbPieces, "pbSetMarkerB");
  pbSetMarkerB->setPixmap(pixMaps[MARKER_B]);


  // Get horizontal box for colors and selected piece
  QHBox *hbColors = new QHBox( vbPieces );

  // Get new group box
  QGroupBox *gbColors = new QGroupBox(2, QGroupBox::Horizontal, tr("Color"), hbColors);
  // Get button for white 
  pbSetWhite = new QPushButton(gbColors, "pbSetWhite");
  pbSetWhite->setPixmap(pixMaps[0]);
  // Get button for black 
  pbSetBlack = new QPushButton(gbColors, "pbSetBlack");
  pbSetBlack->setPixmap(pixMaps[34]);

  // Get new group box
  QGroupBox *gbSelected = new QGroupBox(1, QGroupBox::Horizontal, tr("Selected"), hbColors);
  // Get button for selected piece 
  pbSelectedPiece = new QPushButton(gbSelected, "pbSelectedPiece");
  pbSelectedPiece->setPixmap(pixMaps[PAWN]);

  // Get new group box for text commands
  QGroupBox *gbTextCommands = new QGroupBox(4, QGroupBox::Vertical, tr("Text Commands"), vbPieces);
  pbBoldFaceCommand = new QPushButton(tr("Bold Face"), gbTextCommands, "pbBoldFaceCommand");
  pbItalicFaceCommand = new QPushButton(tr("Italic Face"), gbTextCommands, "pbItalicFaceCommand");
  pbMoveCommand = new QPushButton(tr("Move"), gbTextCommands, "pbMoveCommand");

  // Checkbox for diagram on/off
  cbDiagram = new QCheckBox(tr("Output Diagram"), vbPieces, "cbDiagram");
  
  // Get new vertical box for informator commands
  QGroupBox *gbInformatorCommands = new QGroupBox(3, QGroupBox::Vertical, tr("Informator Symbols"), hbButtons);
  lbInformator = new QListBox( gbInformatorCommands, "lbInformator" );
  cbTrailingSpace = new QCheckBox(tr("With trailing space"), gbInformatorCommands, "cbTrailingSpace");
  pbInformatorCommand = new QPushButton(tr("Insert"), gbInformatorCommands, "pbInformatorCommand");
  lbInformator->insertItem(qApp->translate("ctString", "mate"));
  lbInformator->insertItem(qApp->translate("ctString", "white stands slightly better"));
  lbInformator->insertItem(qApp->translate("ctString", "black stands slightly better"));
  lbInformator->insertItem(qApp->translate("ctString", "white has the upper hand"));
  lbInformator->insertItem(qApp->translate("ctString", "black has the upper hand"));
  lbInformator->insertItem(qApp->translate("ctString", "white has a decisive advantage"));
  lbInformator->insertItem(qApp->translate("ctString", "black has a decisive advantage"));
  lbInformator->insertItem(qApp->translate("ctString", "even"));
  lbInformator->insertItem(qApp->translate("ctString", "unclear"));
  lbInformator->insertItem(qApp->translate("ctString", "with compensation for the material"));
  lbInformator->insertItem(qApp->translate("ctString", "development advantage"));
  lbInformator->insertItem(qApp->translate("ctString", "greater board room"));
  lbInformator->insertItem(qApp->translate("ctString", "with attack"));
  lbInformator->insertItem(qApp->translate("ctString", "with initiative"));
  lbInformator->insertItem(qApp->translate("ctString", "with counter-play"));
  lbInformator->insertItem(qApp->translate("ctString", "zugzwang"));
  lbInformator->insertItem(qApp->translate("ctString", "with the idea"));
  lbInformator->insertItem(qApp->translate("ctString", "only move"));
  lbInformator->insertItem(qApp->translate("ctString", "better is"));
  lbInformator->insertItem(qApp->translate("ctString", "line"));
  lbInformator->insertItem(qApp->translate("ctString", "diagonal"));
  lbInformator->insertItem(qApp->translate("ctString", "centre"));
  lbInformator->insertItem(qApp->translate("ctString", "king's side"));
  lbInformator->insertItem(qApp->translate("ctString", "queen's side"));
  lbInformator->insertItem(qApp->translate("ctString", "weak point"));
  lbInformator->insertItem(qApp->translate("ctString", "ending"));
  lbInformator->insertItem(qApp->translate("ctString", "pair of bishops"));
  lbInformator->insertItem(qApp->translate("ctString", "bishops of opposite color"));
  lbInformator->insertItem(qApp->translate("ctString", "bishops of the same color"));
  lbInformator->insertItem(qApp->translate("ctString", "united pawns"));
  lbInformator->insertItem(qApp->translate("ctString", "separated pawns"));
  lbInformator->insertItem(qApp->translate("ctString", "double pawns"));
  lbInformator->insertItem(qApp->translate("ctString", "passed pawn"));
  lbInformator->insertItem(qApp->translate("ctString", "advantage in number of pawns"));
  lbInformator->insertItem(qApp->translate("ctString", "time"));
  lbInformator->insertItem(qApp->translate("ctString", "a novelty"));
  lbInformator->insertItem(qApp->translate("ctString", "editorial comment"));
  lbInformator->insertItem(qApp->translate("ctString", "various moves"));
  lbInformator->insertItem(qApp->translate("ctString", "without"));
  lbInformator->insertItem(qApp->translate("ctString", "with"));
  lbInformator->insertItem(qApp->translate("ctString", "etc"));
  lbInformator->insertItem(qApp->translate("ctString", "see"));
  lbInformator->insertItem(qApp->translate("ctString", "Marker A (cross)"));
  lbInformator->insertItem(qApp->translate("ctString", "Marker B (circle)"));

  // Get new group box
  QGroupBox *gbTheTask = new QGroupBox(1, QGroupBox::Horizontal, tr("Task"), this);
  // Get multiline edit for task
#ifdef QTVER2X
  mleTask = new QMultiLineEdit(gbTheTask, "mleTask");
#else
  mleTask = new QTextEdit(gbTheTask, "mleTask");
  mleTask->setMinimumHeight(20);
#endif
  // Get new group box
  QGroupBox *gbTheSolution = new QGroupBox(1, QGroupBox::Horizontal, tr("Solution"), this);
  // Get multiline edit for solution
#ifdef QTVER2X
  mleSolution = new QMultiLineEdit(gbTheSolution, "mleSolution");
#else
  mleSolution = new QTextEdit(gbTheSolution, "mleSolution");
  mleSolution->setMinimumHeight(20);
#endif

  // Add keyboard accelerators
  QAccel *accel = new QAccel( this );
  accel->connectItem(accel->insertItem(CTRL+Key_B),
                     this, SLOT( slotInsertBoldFaceCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_I),
                     this, SLOT( slotInsertItalicFaceCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_M),
                     this, SLOT( slotInsertMoveCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_L),
                     this, SLOT( slotInsertLaTeXHTMLCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_S),
                     this, SLOT( slotInsertInformatorCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_K),
                     this, SLOT( slotInsertKnightMoveCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_A),
                     this, SLOT( slotInsertArrowCommand() ));
  accel->connectItem(accel->insertItem(CTRL+Key_H),
                     this, SLOT( slotInsertHighlightCommand() ));

}

/** Sets up all necessary ``connects'' between \a signals and \a slots 
for the widget. */
void ctOption::setupConnects()
{

/****************************/
/* Signal router -> Options */
/****************************/
  connect(srChessTask, SIGNAL( sigOpenFile() ),
          this, SLOT( slotOpenFile() ));
  connect(srChessTask, SIGNAL( sigAddFile() ),
          this, SLOT( slotAddFile() ));
  connect(srChessTask, SIGNAL( sigSaveFile() ),
          this, SLOT( slotSaveFile() ));
  connect(srChessTask, SIGNAL( sigExportLaTeXFile() ),
          this, SLOT( slotExportLaTeXFile() ));
  connect(srChessTask, SIGNAL( sigExportChessCardsFile() ),
          this, SLOT( slotExportChessCardsFile() ));
  connect(srChessTask, SIGNAL( sigExportHTML() ),
          this, SLOT( slotExportHTML() ));
  connect(srChessTask, SIGNAL( sigExportLaTeXDirectory() ),
          this, SLOT( slotExportLaTeXDirectory() ));
  connect(srChessTask, SIGNAL( sigExportChessCardsDirectory() ),
          this, SLOT( slotExportChessCardsDirectory() ));
  connect(srChessTask, SIGNAL( sigExportHTMLDirectory() ),
          this, SLOT( slotExportHTMLDirectory() ));
  connect(srChessTask, SIGNAL( sigCopyLaTeXPosition() ),
          this, SLOT( slotCopyLaTeXPosition() ));
  connect(srChessTask, SIGNAL( sigCopyHTMLPosition() ),
          this, SLOT( slotCopyHTMLPosition() ));
  connect(srChessTask, SIGNAL( sigImportFENFile() ),
          this, SLOT( slotImportFENFile() ));
  connect(srChessTask, SIGNAL( sigImportEPDFile() ),
          this, SLOT( slotImportEPDFile() ));
  connect(srChessTask, SIGNAL( sigDeleteAll() ),
          this, SLOT( slotDeleteAll() ));
  connect(srChessTask, SIGNAL( sigInsertTask() ),
          this, SLOT( slotInsertTask() ));
  connect(srChessTask, SIGNAL( sigCopyTask() ),
          this, SLOT( slotCopyTask() ));
  connect(srChessTask, SIGNAL( sigDeleteTask() ),
          this, SLOT( slotDeleteTask() ));
  connect(srChessTask, SIGNAL( sigTaskBack() ),
          this, SLOT( slotTaskBack() ));
  connect(srChessTask, SIGNAL( sigTaskForward() ),
          this, SLOT( slotTaskForward() ));
  connect(srChessTask, SIGNAL( sigFirstTask() ),
          this, SLOT( slotFirstTask() ));
  connect(srChessTask, SIGNAL( sigLastTask() ),
          this, SLOT( slotLastTask() ));
  connect(srChessTask, SIGNAL( sigSetMainTitle() ),
          this, SLOT( slotSetMainTitle() ));
  connect(srChessTask, SIGNAL( sigGenerateTitles() ),
          this, SLOT( slotGenerateTitles() ));
  connect(srChessTask, SIGNAL( sigRestoreTask() ),
          this, SLOT( slotRestoreTask() ));

  connect(srChessTask, SIGNAL( sigSetDirLevel(int) ),
          this, SLOT( slotSetDirLevel(int) ));

/**********/
/* Pieces */
/**********/
    
    connect( pbSetPawn, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToPawn() ));
    connect( pbSetKnight, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToKnight() ));
    connect( pbSetBishop, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToBishop() ));
    connect( pbSetRook, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToRook() ));
    connect( pbSetQueen, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToQueen() ));
    connect( pbSetKing, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToKing() ));
    connect( pbSetMarkerA, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToMarkerA() ));
    connect( pbSetMarkerB, SIGNAL( clicked() ),
	     this, SLOT( slotPieceChangedToMarkerB() ));
    
    connect( pbBoldFaceCommand, SIGNAL( clicked() ),
	     this, SLOT( slotInsertBoldFaceCommand() ));
    connect( pbItalicFaceCommand, SIGNAL( clicked() ),
	     this, SLOT( slotInsertItalicFaceCommand() ));
    connect( pbMoveCommand, SIGNAL( clicked() ),
	     this, SLOT( slotInsertMoveCommand() ));
    connect( pbInformatorCommand, SIGNAL( clicked() ),
	     this, SLOT( slotInsertInformatorCommand() ));

/**********/
/* Colors */ 
/**********/
 
    connect( pbSetWhite, SIGNAL( clicked() ),
	     this, SLOT( slotColorChangedToWhite() ));
    connect( pbSetBlack, SIGNAL( clicked() ),
	     this, SLOT( slotColorChangedToBlack() ));

/*******************************/
/* Signal router -> Task list  */
/*******************************/

  connect(srChessTask, SIGNAL( sigSetA4PaperSizeEnable(bool) ),
          tlTasks, SLOT( slotSetA4PaperSizeEnable(bool) ));
  connect(srChessTask, SIGNAL( sigSetLaTeXDiagramSize(int) ),
          tlTasks, SLOT( slotSetLaTeXDiagramSize(int) ));
  connect(srChessTask, SIGNAL( sigSetTwoColumnsEnable(bool) ),
          tlTasks, SLOT( slotSetTwoColumnsEnable(bool) ));
  connect(srChessTask, SIGNAL( sigSetOneFileEnable(bool) ),
          tlTasks, SLOT( slotSetOneFileEnable(bool) ));
  connect(srChessTask, SIGNAL( sigSetUseTTFontEnable(bool) ),
          tlTasks, SLOT( slotSetUseTTFontEnable(bool) ));
  connect(srChessTask, SIGNAL( sigSetExpandSymbolsEnable(bool) ),
          tlTasks, SLOT( slotSetExpandSymbolsEnable(bool) ));
  connect(srChessTask, SIGNAL( sigSetOutputSolutionsEnable(bool) ),
          tlTasks, SLOT( slotSetOutputSolutionsEnable(bool) ));
  connect(srChessTask, SIGNAL( sigSetFontSize(int) ),
          tlTasks, SLOT( slotSetFontSize(int) ));

/**********************/
/* Board -> Task list */
/**********************/

  connect(dbDrawingBoard, SIGNAL( sigSetPosition(const ctPosition &) ),
          tlTasks, SLOT( slotSetPosition(const ctPosition &) ));
}    
    
/** Finds all ``*.tsk'' files in the given directory and its
subdirectories and returns the number of files.
@param searchDir The path of the directory
@return Number of found ``*.tsk'' files
*/
int ctOption::getNumberOfTaskFiles(const QString &searchDir)
{
  // Counter
  int sum = 0;

  // Directory "*.tsk" file entries
  QDir taskFiles(searchDir, "*.tsk");
  sum += taskFiles.count();

  // Subdirectories
  QDir subDirectories(searchDir);
  
  subDirectories.setFilter(QDir::Dirs | QDir::NoSymLinks);

  // Create list of file info entries
  const QFileInfoList *list = subDirectories.entryInfoList();
  // Create list iterator
  QFileInfoListIterator l_it(*list);
  // Pointer to a single file info entry
  QFileInfo *fi;

  while ((fi = l_it.current()))
  {
    if ((fi->fileName() != ".") && (fi->fileName() != ".."))
    {  
      sum += getNumberOfTaskFiles(fi->absFilePath());
    }
    ++l_it;
  }  

  return(sum);
}


/** Recursively scans the given directory \a sourceDirName for ``ChessTask''
files with the extension \c *.tsk. The subdirectories of \a sourceDirName
are created in \a destDirName too and every task file is exported to the
equivalent directory. Additionally a \c Makefile is created in each
subdirectory. It calls \c LaTeX and \c dvips for automatically generating
\a PostScript files. Depending on \a exportType either ``normal''
LaTeX files (for ``LATEX'') or files in the ``ChessCards'' format (for ``CARDS'')
are exported.
@param exportType Can be either ``LATEX'' for normal LaTeX files or
``CARDS'' for ChessCards format.
@param sourceDirName The source directory
@param destDirName The destination directory all files are exported to
@param fileNumber Counter for the number of already processed files, used
for the progress bar
@see directoryExport
*/
void ctOption::recursiveLaTeXDirectoryExport(int exportType,
                                             const QString &sourceDirName, 
                                             const QString &destDirName,
					     int &fileNumber)
{

  // Directory "*.tsk" file entries
  QDir taskFiles(sourceDirName, "*.tsk");
  taskFiles.setFilter(QDir::Files | QDir::NoSymLinks);
  taskFiles.setSorting(QDir::Name);

  // Create list of file info entries
  const QFileInfoList *t_list = taskFiles.entryInfoList();
  // Create list iterator
  QFileInfoListIterator tl_it(*t_list);
  // Pointer to a single file info entry
  QFileInfo *fi;

  // File for reading/writing
  QFile f;
  // Text stream
  QTextStream t;

  // Process ``*.tsk'' files in the given directory
  while ((fi = tl_it.current()))
  {
    // Open file for reading
    // Set file name
    f.setName(sourceDirName + fi->fileName());
    if (f.open(IO_ReadOnly))
    {
      // Set text stream to opened file
      t.setDevice( &f );

      // Delete list of tasks
      tlExport->slotDeleteAll();
      // Read file
      tlExport->slotReadFile(t);

      // Close file
      f.close();

      if (tlExport->slotGetSize() > 0)
      {
	// Export tasks
        // Set file name for writing...
        f.setName(destDirName + fi->baseName() + ".tex");
        if (f.open(IO_WriteOnly))
        {
          t.setDevice( &f );

          // Write file name
          t << "%% " << destDirName + fi->baseName() + ".tex" << endl;
          
          if (exportType == LATEX)
            tlExport->slotExportLaTeXFile(t);
          else
            tlExport->slotExportChessCardsFile(t);
      
          // Close file
          f.close();
	}
      } // End of: if (tlExport->slotGetSize() > 0)	
    } // End of: if (f.open(IO_ReadOnly))

    // Increase number of processed files
    fileNumber++;
    pdProgress->slotSetCurrentProgress(fileNumber);

    ++tl_it;
  } // End of: while (l_it = fi.current())


  // Process subdirectories
  QDir subDirectories(sourceDirName);
  subDirectories.setFilter(QDir::Dirs | QDir::NoSymLinks);
  subDirectories.setSorting(QDir::Name);
  // Create list of file info entries
  const QFileInfoList *d_list = subDirectories.entryInfoList();
  // Create list iterator
  QFileInfoListIterator dl_it(*d_list);

  while ((fi = dl_it.current()))
  {
    if ((fi->fileName() != ".") && (fi->fileName() != ".."))
    {  
      // Check if destination directory exists
      if (subDirectories.exists(destDirName + fi->fileName()) == false)
      {
        // No, so create directory
	if (subDirectories.mkdir(destDirName + fi->fileName()) == false)
	{
	  break;
	}
      }	
      
      // Recursive call for the new directory
      recursiveLaTeXDirectoryExport(exportType,
                                    sourceDirName + fi->fileName() + "/",
                                    destDirName + fi->fileName() + "/",
				    fileNumber);
    }
    ++dl_it;
  }  

  // Write makefile
  f.setName(destDirName + "Makefile");
  if (f.open(IO_WriteOnly))
  {
    // Set text stream
    t.setDevice( &f );

    // Write Makefile header
    t << "# Created by ChessTask v2.0" << endl;

    // Write list of PostScript files
    t << "PSFILES=";

    if ((*t_list).count() > 0)
    {
      // Reset iterator for task files
      fi = tl_it.toFirst();
      t << fi->baseName() << ".ps";
      ++tl_it;

      while ((fi = tl_it.current()))
      {
        t << "\\\n        ";
        t << fi->baseName() << ".ps";
        ++tl_it;
      }
    } 

    t << endl << endl;

    // Write list of PDF files
    t << "PDFFILES=";

    if ((*t_list).count() > 0)
    {
      // Reset iterator for task files
      fi = tl_it.toFirst();
      t << fi->baseName() << ".pdf";
      ++tl_it;

      while ((fi = tl_it.current()))
      {
        t << "\\\n         ";
        t << fi->baseName() << ".pdf";
        ++tl_it;
      }
    } 

    // Target "all" creating PS files
    t << endl << endl << "all: $(PSFILES)" << endl;

    // Reset iterator for directories
    dl_it.toFirst();

    while ((fi = dl_it.current()))
    {
      if ((fi->fileName() != ".") && (fi->fileName() != ".."))
      {  
        t << "\tmake -C " << fi->fileName() << ";" << endl;
      }	
      ++dl_it;
    }
    
    // Target "pdf" creating PDF files
    t << endl << endl << "pdf: $(PDFFILES)" << endl;

    // Reset iterator for directories
    dl_it.toFirst();

    while ((fi = dl_it.current()))
    {
      if ((fi->fileName() != ".") && (fi->fileName() != ".."))
      {  
        t << "\tmake -C " << fi->fileName() << " pdf;" << endl;
      }	
      ++dl_it;
    }

    // Implicit rule for PDF files
    t << endl << "%.pdf : %.ps" << endl;
    t << "\tps2pdf -r600 ";
    if (tlTasks->getA4PaperSizeEnabled() == true)
      t << "-sPAPERSIZE=a4";
    else
      t << "-sPAPERSIZE=letter";
    t << " $<" << endl << endl;

    // Implicit rule for PS files
    t << "%.ps : %.dvi" << endl;
    t << "\tdvips -D600 ";
    if (tlTasks->getA4PaperSizeEnabled() == true)
      t << "-t a4";
    else
      t << "-t letter";
    t << " -o $@ $<" << endl << endl;
    t << "%.dvi : %.tex" << endl;
    t << "\tlatex $*" << endl << endl;

    // Close file 
    f.close();
  }  

}  

/** Recursively scans the given directory \a sourceDirName for ``ChessTask''
files with the extension \c *.tsk. The subdirectories of \a sourceDirName
are created in \a destDirName too and every task file is exported to an
equivalent directory of the same name. 
@param sourceDirName The source directory
@param destDirName The destination directory all files are exported to
@param fileNumber Counter for the number of already processed files, used
for the progress bar
@see directoryExport
*/
void ctOption::recursiveHTMLDirectoryExport(const QString &sourceDirName, 
                                            const QString &destDirName,
                                            QStringList &slIndex,
					    int &fileNumber)
{
  // Pointer to a single file info entry
  QFileInfo *fi;

  // File for reading/writing
  QFile f;
  // Text stream
  QTextStream t;

  // Current path to the ``graphics directory''
  QString graphDir;
  // Counter
  unsigned int i;
  for (i = 0; i < slIndex.count()+2; i++)
  {
    graphDir += "../";
  }
  graphDir += "graphics/";

  // ``Yahoo'' index string to the directories above
  QString indexString = "<A HREF=\"";
  // Construct link to global index
  for (i = 0; i < slIndex.count()+1; i++)
  {
    indexString += "../";
  }
  indexString += "index.html\">" + tr("Global index") + "</A>\n";
  // Additional counter
  unsigned int j;
  for (i = 0; i < slIndex.count(); i++)
  {
    indexString += "&gt;&gt; <A HREF=\"";
    for (j = slIndex.count() - i; j > 0; j--)
    {
      indexString += "../";
    }
    indexString += "index.html\">" + slIndex[i] + "</A>\n";
  }

  // Process subdirectories
  QDir subDirectories(sourceDirName);
  subDirectories.setFilter(QDir::Dirs | QDir::NoSymLinks);
  subDirectories.setSorting(QDir::Name);
  // Create list of file info entries
  const QFileInfoList *d_list = subDirectories.entryInfoList();
  // Create list iterator
  QFileInfoListIterator dl_it(*d_list);

  while ((fi = dl_it.current()))
  {
    if ((fi->fileName() != ".") && (fi->fileName() != ".."))
    {  
      // Check if destination directory exists
      if (subDirectories.exists(destDirName + fi->fileName()) == false)
      {
        // No, so create directory
	if (subDirectories.mkdir(destDirName + fi->fileName()) == false)
	{
	  break;
	}
      }	
      // Append directory name to index list
      slIndex.append(fi->fileName());
      // Recursive call for the new directory
      recursiveHTMLDirectoryExport(sourceDirName + fi->fileName() + "/",
                                   destDirName + fi->fileName() + "/",
				   slIndex, fileNumber);
      // Remove last directory name from index list
      slIndex.remove(slIndex.fromLast());
    }
    ++dl_it;
  }  


  // Directory "*.tsk" file entries
  QDir taskFiles(sourceDirName, "*.tsk");
  taskFiles.setFilter(QDir::Files | QDir::NoSymLinks);
  taskFiles.setSorting(QDir::Name);

  // Create list of file info entries
  const QFileInfoList *t_list = taskFiles.entryInfoList();
  // Create list iterator
  QFileInfoListIterator tl_it(*t_list);

  // List of main titles
  QStringList slMainTitles;

  // Process ``*.tsk'' files in the given directory
  while ((fi = tl_it.current()))
  {
    // Open file for reading
    // Set file name
    f.setName(sourceDirName + fi->fileName());
    if (f.open(IO_ReadOnly))
    {
      // Set text stream to opened file
      t.setDevice( &f );

      // Delete list of tasks
      tlExport->slotDeleteAll();
      // Read file
      tlExport->slotReadFile(t);

      // Close file
      f.close();

      // Append main title to list for the index
      if (tlExport->slotGetMainTitle().isEmpty())
      {
        slMainTitles += tr("No title") + " (" + 
	                tr("File") + ": " + fi->fileName() + ")";
      }
      else
      {
        slMainTitles += tlExport->slotGetMainTitle();
      }

      if (tlExport->slotGetSize() > 0)
      {
	// Export tasks
        // Check if destination directory exists
        if (taskFiles.exists(destDirName + fi->baseName()) == false)
        {
          // No, so create directory
	  if (taskFiles.mkdir(destDirName + fi->baseName()) == false)
	  {
	    break;
	  }
        }
        tlExport->slotExportHTML(destDirName + fi->baseName(), 
	                         graphDir, indexString);
      }
    } // End of: if (f.open(IO_ReadOnly))
    else
    {
      slMainTitles += tr("Could not be opened") + " (" + 
                      tr("File") + ": " + fi->fileName() + ")";
    }
    
    // Increase number of processed files
    fileNumber++;
    pdProgress->slotSetCurrentProgress(fileNumber);

    ++tl_it;
  } // End of: while (l_it = fi.current())


  // Write index file for current directory
  f.setName(destDirName + "index.html");
  if (f.open(IO_WriteOnly))
  {
    // Set text stream
    t.setDevice( &f );

    // Write header
    if (slIndex.count() > 0)
    {
      writeHTMLHeader(t, tr("Index of: ") + slIndex.last(),
                      tlTasks->getUseTTFontEnabled());

      // ``Yahoo'' index string to the directories above
      t << "<A HREF=\"";
      // Construct link to global index
      for (i = 0; i < slIndex.count(); i++)
      {
        t << "../";
      }
      t << "index.html\">" + tr("Global index") + "</A>\n";
      for (i = 0; i < (slIndex.count() - 1); i++)
      {
        t << "&gt;&gt; <A HREF=\"";
        for (j = slIndex.count() - i - 1; j > 0; j--)
        {
          t << "../";
        }
        t << "index.html\">" + slIndex[i] + "</A>\n";
      }

      t << endl << "<P>" << endl;
    
      // Write heading
      t << "<H1>" + tr("Index of: ") + slIndex.last() << "</H1>" << endl;
    }
    else
    {
      writeHTMLHeader(t, tr("Global index"), 
                      tlTasks->getUseTTFontEnabled());

      // Write heading
      t << "<H1>" + tr("Global index") << "</H1>" << endl;
    }  

    // Correct path to ``graphics'' directory
    graphDir.remove(0, 3);

    // Check if we have any directories
    // The directories ``.'' and ``..'' always
    // exist, that's why the number should be > 2 
    if ((*d_list).count() > 2)
    {
      t << "<P>\n<H3>" << tr("Directories") << "</H3>" << endl;

      // Write directory entries
      dl_it.toFirst();
      while ((fi = dl_it.current()))
      {
        if ((fi->fileName() != ".") && (fi->fileName() != ".."))
        {  
          t << "<A HREF=\"./" << fi->fileName() << "/index.html\">";
	  t << fi->fileName() << "</A><BR>" << endl;
        }
      
        ++dl_it;
      }  
    }

    // Check if we have any task files
    if ((*t_list).count() > 0)
    {
      t << "<P>\n<H3>" << tr("Task files") << "</H3>" << endl;

      // Write task entries
      tl_it.toFirst();
      i = 0;
      while ((fi = tl_it.current()))
      {
        t << "<A HREF=\"./" << fi->baseName() << "/index.html\">";
        t << slMainTitles[i] << "</A><BR>" << endl;
      
        i++;
        ++tl_it;
      }  
    }

    // Write trailer
    writeHTMLTrailer(t, graphDir, tlTasks->getUseTTFontEnabled()); 
    // Close file
    f.close();
  }  

}  

/** Exports all task files in the
specified directory, into one or more directories, 
containing the tasks and the solutions.
@param exportType This variable has to be set
to either ``LATEX'', ``CARDS'' or ``HTML'', depending on what format
should be exported.
@see recursiveLaTeXDirectoryExport, recursiveHTMLDirectoryExport
*/
void ctOption::directoryExport(int exportType)
{

  QString sourceDirName = QFileDialog::getExistingDirectory();
  if ( !sourceDirName.isNull() ) 
  { 
    // Show info message
    QMessageBox::information(0, 
                             tr("Information"), 
			     tr("You successfully selected the source directory!") + "\n\n" +
                             tr("Next, you have to specify the directory where all") + "\n" +
			     tr("files should be exported to."));
    
    QString destDirName = QFileDialog::getExistingDirectory();
    
    if ( !destDirName.isNull() )
    {

      // We got the two directories, so write the tasks...

      // Ensure that both directories end with a "/"
      if (sourceDirName.at(sourceDirName.length()-1) != '/')
      {
        sourceDirName += "/";
      }	
      if (destDirName.at(destDirName.length()-1) != '/')
      {
        destDirName += "/";
      }	

      // Get number of files to export
      pdProgress->slotSetTotalProgressSteps(getNumberOfTaskFiles(sourceDirName));

      // Task list for the export
      tlExport = new ctTaskList;
      // Copy options from the current list of tasks
      tlExport->getOptions(*tlTasks);

      // Reset and show progress window
      pdProgress->slotResetProgressBar();
      pdProgress->show();

      int fileNumber = 0;

      if (exportType == LATEX)
      {
        recursiveLaTeXDirectoryExport(LATEX, sourceDirName, destDirName, 
	                              fileNumber);
      }
      if (exportType == CARDS)
      {
        recursiveLaTeXDirectoryExport(CARDS, sourceDirName, destDirName, 
	                              fileNumber);
      }
      if (exportType == HTML)
      {
        QStringList slIndex;

        recursiveHTMLDirectoryExport(sourceDirName, destDirName,
                                     slIndex, fileNumber);
      }

      // Hide progress window again
      pdProgress->hide();
  
      delete tlExport;
    }
  }

}


/*---------------------------------------------------------------- Slots */

/**********/
/* Public */
/**********/


/** Opens a ``ChessTask'' file and tries to read from it.
The current list of tasks is deleted before reading the new data.
@see slotAddFile
*/
void ctOption::slotOpenFile()
{
  QString fileName = QFileDialog::getOpenFileName();

  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so read from file...
    QFile f(fileName);
    if (f.open(IO_ReadOnly))
    {
      QTextStream t( &f );

      // Delete list of tasks
      slotDeleteAll();
      // Read file
      tlTasks->slotReadFile(t);

      // Close file
      f.close();

      if (tlTasks->slotGetSize() > 0)
      {
	// Enable board
	dbDrawingBoard->slotSetBoardEnable(true);
        // Restore the current task
        slotRestoreTask();
      }	
    }
     
  }
}

/** This slot function opens a ``ChessTask'' file and tries to read from it.
The current list of tasks is not deleted before reading the new data, so the
tasks are appended to the list.
@see slotOpenFile
*/
void ctOption::slotAddFile()
{
  QString fileName = QFileDialog::getOpenFileName();

  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so read from file...
    QFile f(fileName);
    if (f.open(IO_ReadOnly))
    {
      QTextStream t( &f );

      // Read file
      tlTasks->slotReadFile(t);

      // Close file
      f.close();

      if (tlTasks->slotGetSize() > 0)
      {
	// Enable board
	dbDrawingBoard->slotSetBoardEnable(true);
        // Restore the current task
        slotRestoreTask();
      }	
    }
     
  }
}

/** Opens a file for writing and saves the list of tasks in it.
*/
void ctOption::slotSaveFile()
{
  if (tlTasks->slotGetSize() == 0)
  {
    return;
  }  

  QString fileName = QFileDialog::getSaveFileName();
  if ( !fileName.isNull() ) 
  {                 
    // Store the current task
    slotSaveTask();

    // We got a file name, so write the file...
    QFile f(fileName);
    if (f.open(IO_WriteOnly))
    {
      QTextStream t( &f );
      
      tlTasks->slotSaveFile(t);

      // Close file
      f.close();
    }
  }

}

/** Opens a FEN file and inserts the found positions
in the ``Forsyth Edwards Notation'' into the list of tasks.
*/
void ctOption::slotImportFENFile()
{
  QString fileName = QFileDialog::getOpenFileName();

  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so read from file...
    QFile f(fileName);
    if (f.open(IO_ReadOnly))
    {
      QTextStream t( &f );

      tlTasks->slotImportFENFile(t);
      
      f.close();

      if (tlTasks->slotGetSize() > 0)
      {
	// Enable board
	dbDrawingBoard->slotSetBoardEnable(true);
        // Restore the current task
        slotRestoreTask();
      }	
    }
     
  }

}

/** Opens an EPD (Extended Position Description)
file and inserts the found positions into the list of tasks.
*/
void ctOption::slotImportEPDFile()
{
  QString fileName = QFileDialog::getOpenFileName();

  if ( !fileName.isNull() ) 
  {                 
    // We got a file name, so read from file...
    QFile f(fileName);
    if (f.open(IO_ReadOnly))
    {
      QTextStream t( &f );

      tlTasks->slotImportEPDFile(t);
      
      f.close();

      if (tlTasks->slotGetSize() > 0)
      {	
	// Enable board
	dbDrawingBoard->slotSetBoardEnable(true);
        // Restore the current task
        slotRestoreTask();
      }	
    }
     
  }

}

/** Opens a file for writing and exports a LaTeX file,
containing the tasks and the solutions.
*/
void ctOption::slotExportLaTeXFile()
{
  if (tlTasks->slotGetSize() == 0)
  {
    return;
  }  

  QString fileName = QFileDialog::getSaveFileName();
  if ( !fileName.isNull() ) 
  {                 
    // Store the current task
    slotSaveTask();

    // We got a file name, so write the file...
    QFile f(fileName);
    if (f.open(IO_WriteOnly))
    {
      QTextStream t( &f );

      // Write file name
      t << "%% " << fileName << endl;
      tlTasks->slotExportLaTeXFile(t);
      
      // Close file
      f.close();
    }
  }

}

/** Opens a file for writing and exports a LaTeX file,
containing the tasks and solutions as ``ChessCards''.
*/
void ctOption::slotExportChessCardsFile()
{
  if (tlTasks->slotGetSize() == 0)
  {
    return;
  }  

  QString fileName = QFileDialog::getSaveFileName();
  if ( !fileName.isNull() ) 
  {                 
    // Store the current task
    slotSaveTask();

    // We got a file name, so write the file...
    QFile f(fileName);
    if (f.open(IO_WriteOnly))
    {
      QTextStream t( &f );

      // Write file name
      t << "%% " << fileName << endl;
      tlTasks->slotExportChessCardsFile(t);
      
      // Close file
      f.close();
    }
  }

}

/** Exports the data to one or more
HTML files, containing the tasks and the solutions.
*/
void ctOption::slotExportHTML()
{

  if (tlTasks->slotGetSize() == 0)
  {
    return;
  }  

  QString dirName = QFileDialog::getExistingDirectory();
  if ( !dirName.isNull() ) 
  {                 
    // Store the current task
    slotSaveTask();

    // We got a directory, so write the tasks...

    // Ensure that both directories end with a "/"
    if (dirName.at(dirName.length()-1) != '/')
    {
      dirName += "/";
    }	
    tlTasks->slotExportHTML(dirName, graphicsDir);
  }

}

/** Exports all task files in the
specified directory, into one or more
LaTeX files, containing the tasks and the solutions.
@see directoryExport
*/
void ctOption::slotExportLaTeXDirectory()
{

  directoryExport(LATEX);

}

/** Exports all task files in the
specified directory, into one or more
LaTeX files, containing the tasks and solutions
as ``ChessCards''.
@see directoryExport
*/
void ctOption::slotExportChessCardsDirectory()
{

  directoryExport(CARDS);

}

/** Exports all task files in the
specified directory, into one or more directories with
HTML files, containing the tasks and the solutions.
@see directoryExport
*/
void ctOption::slotExportHTMLDirectory()
{

  directoryExport(HTML);

}

/** Copies the current position to the application's
clipboard in LaTeX format. From there it can be pasted
into other applications, e.g. a text editor.
*/
void ctOption::slotCopyLaTeXPosition()
{
  dbDrawingBoard->slotCopyLaTeXPosition();
}

/** Copies the current position to the application's
clipboard in HTML format. From there it can be pasted
into other applications, e.g. a text editor.
*/
void ctOption::slotCopyHTMLPosition()
{
  dbDrawingBoard->slotCopyHTMLPosition(graphicsDir);
}

/** Deletes all current data at once.
*/
void ctOption::slotDeleteAll()
{
  if (tlTasks->slotGetSize() == 0)
  {
    return;
  }
  
  // Delete tasks
  tlTasks->slotDeleteAll();
  leTitle->setText("");
  mleTask->setText("");
  mleSolution->setText("");
  cbDiagram->setChecked(true);
  // Disable board
  dbDrawingBoard->slotSetBoardEnable(false);
}

/** Inserts a new task into the list.
*/
void ctOption::slotInsertTask()
{
  if (tlTasks->slotGetSize() > 0)
  {
    slotSaveTask();
  }
  else
  {
    dbDrawingBoard->slotSetBoardEnable(true);
  }  
    
  tlTasks->slotInsertTask();
  slotRestoreTask();

}

/** Inserts a new task into the list at the
actual position and copies the diagram.
*/
void ctOption::slotCopyTask()
{
  if (tlTasks->slotGetSize() > 0)
  {
    // Store actual task
    slotSaveTask();
    tlTasks->slotCopyTask();
    slotRestoreTask();
  }

}

/** Deletes the current task.
*/
void ctOption::slotDeleteTask()
{
  if (tlTasks->slotGetSize() > 0)
  {
    // Erase element
    tlTasks->slotDeleteTask();
    
    // Is the list not empty?
    if (tlTasks->slotGetSize() > 0) 
    {
      slotRestoreTask();
    }
    else
    {
      leTitle->setText("");
      mleTask->setText("");
      mleSolution->setText("");
      cbDiagram->setChecked(true);
      // Disable board
      dbDrawingBoard->slotSetBoardEnable(false);

    }  
  }  
}

/** Moves one step back in the list of tasks.
*/
void ctOption::slotTaskBack()
{
  if (tlTasks->slotGetSize() > 0)
  {
    slotSaveTask();
    tlTasks->slotTaskBack();
    slotRestoreTask();
  }  
}

/** Moves one step forward in the list of tasks.
*/
void ctOption::slotTaskForward()
{
  if (tlTasks->slotGetSize() > 0)
  {
    slotSaveTask();
    tlTasks->slotTaskForward();
    slotRestoreTask();
  }  
}

/** Moves to the first entry in the list of tasks.
*/
void ctOption::slotFirstTask()
{
  if (tlTasks->slotGetSize() > 0)
  {
    slotSaveTask();
    tlTasks->slotFirstTask();
    slotRestoreTask();
  }  
}

/** Moves to the last entry in the list of tasks.
*/
void ctOption::slotLastTask()
{
  if (tlTasks->slotGetSize() > 0)
  {
    slotSaveTask();
    tlTasks->slotLastTask();
    slotRestoreTask();
  }  
}

/** Restores the current task from the list \a tlTasks.
*/
void ctOption::slotRestoreTask()
{
  // Set the position
  dbDrawingBoard->slotSetPosition(tlTasks->slotGetPosition());
  // Set the task number
  dbDrawingBoard->slotSetTaskNumber(QString::number(tlTasks->slotGetCurrentID()+1) + 
                                    "/" +
                                    QString::number(tlTasks->slotGetSize()));
  // Set the title
  leTitle->setText(tlTasks->slotGetTitleText());
  // Set the task text
  mleTask->setText(tlTasks->slotGetTaskText());
  // Set the solution text
  mleSolution->setText(tlTasks->slotGetSolutionText());
  // Set the check box
  cbDiagram->setChecked(tlTasks->slotGetDiagramEnable());
}

/** Saves the current texts and the position to
the task list. The function calls the \a slotGetPosition slot of the
\a ctDrawBoard \a dbDrawingBoard. Then a \a sigSetPosition
signal is sent to the list of tasks \a tlTasks.
*/
void ctOption::slotSaveTask()
{
  // Set the position
  dbDrawingBoard->slotGetPosition();
  // Set the title
  tlTasks->slotSetTitleText(leTitle->text());
  // Set the task text
  tlTasks->slotSetTaskText(mleTask->text());
  // Set the solution text
  tlTasks->slotSetSolutionText(mleSolution->text());
  // Set the check box status
  tlTasks->slotSetDiagramEnable(cbDiagram->isChecked());
}

/** Sets the main title for the list of tasks.
*/
void ctOption::slotSetMainTitle()
{
  ctTitleDialog st( 0, "mainTitle");

  st.setTitleText(tlTasks->slotGetMainTitle());
  st.exec();
  tlTasks->slotSetMainTitle(st.getTitleText());
}

/** Automatically creates a title for all
existing tasks, starting with the one currently displayed.
*/
void ctOption::slotGenerateTitles()
{
  if (tlTasks->slotGetSize() > 0)
  {
    slotSaveTask();
    tlTasks->slotGenerateTitles();
    slotRestoreTask();
  }  
}

/** Sets the new string for the relative path
to the ``graphics'' directory.
@param depth The new depth of the relative path
*/
void ctOption::slotSetDirLevel(int depth)
{
  // Counter
  int i;
  
  graphicsDir = "";
  for (i = 0; i < depth; i++)
  {
    graphicsDir += "../";
  }

  graphicsDir += "graphics/";
}




/** Is called when the ``pawn'' button was selected.
Calculates the new pixmap ID of the piece from
the pawn ID and the currently set color.
Also the private variable \a currentPiece is set to \a PAWN.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToPawn()
{
  int pixmapID;

  if (currentPiece != PAWN)
  {
    // Set current piece
    pixmapID = currentPiece = PAWN;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }
}



/** Is called when the ``knight'' button was selected.
Calculates the new pixmap ID of the piece from
the numb ID and the currently set color.
Also the private variable \a currentPiece is set to \a KNIGHT.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToKnight()
{
  int pixmapID;

  if (currentPiece != KNIGHT)
  {
    // Set current piece
    pixmapID = currentPiece = KNIGHT;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}

/** Is called when the ``bishop'' button was selected.
Calculates the new pixmap ID of the piece from
the bishop ID and the currently set color.
Also the private variable \a currentPiece is set to \a BISHOP.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToBishop()
{
  int pixmapID;

  if (currentPiece != BISHOP)
  {
    // Set current piece
    pixmapID = currentPiece = BISHOP;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}


/** Is called when the ``rook'' button was selected.
Calculates the new pixmap ID of the piece from
the rook ID and the currently set color.
Also the private variable \a currentPiece is set to \a ROOK.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToRook()
{
  int pixmapID;

  if (currentPiece != ROOK)
  {
    // Set current piece
    pixmapID = currentPiece = ROOK;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}

/** Is called when the ``queen'' button was selected.
Calculates the new pixmap ID of the piece from
the queen ID and the currently set color.
Also the private variable \a currentPiece is set to \a QUEEN.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToQueen()
{
  int pixmapID;

  if (currentPiece != QUEEN)
  {
    // Set current piece
    pixmapID = currentPiece = QUEEN;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}

/** Is called when the ``king'' button was selected.
Calculates the new pixmap ID of the piece from
the king ID and the currently set color.
Also the private variable \a currentPiece is set to \a KING.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToKing()
{
  int pixmapID;

  if (currentPiece != KING)
  {
    // Set current piece
    pixmapID = currentPiece = KING;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}

/** Is called when the ``marker A'' button was selected.
Calculates the new pixmap ID of the piece from
the marker A ID and the currently set color.
Also the private variable \a currentPiece is set to \a MARKER_A.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToMarkerA()
{
  int pixmapID;

  if (currentPiece != MARKER_A)
  {
    // Set current piece
    pixmapID = currentPiece = MARKER_A;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}

/** Is called when the ``marker B'' button was selected.
Calculates the new pixmap ID of the piece from
the marker B ID and the currently set color.
Also the private variable \a currentPiece is set to \a MARKER_B.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotPieceChangedToMarkerB()
{
  int pixmapID;

  if (currentPiece != MARKER_B)
  {
    // Set current piece
    pixmapID = currentPiece = MARKER_B;

    // Correct pixmapID if necessary
    if (currentColor ==  BLACK)
    {
      pixmapID += 8;
    }  

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[pixmapID]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(pixmapID);
  }  

}


/** Is called when the color should change to white.
Calculates the new pixmap ID of the piece from
the current piece and the new color \a WHITE, if it was not set before.
Also the private variable \a currentColor is set to \a WHITE.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotColorChangedToWhite()
{
  if (WHITE != currentColor)
  {
    // Set current color
    currentColor = WHITE;

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[currentPiece]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(currentPiece);
  }

}


/** Is called when the color should change to black.
Calculates the new pixmap ID of the piece from
the current piece and the new color \a BLACK, if it was not set before.
Also the private variable \a currentColor is set to \a BLACK.
The slot \a slotSetPiece of the \a ctDrawBoard is called
and the currently selected
piece is displayed in the button \a pbSelectedPiece.
*/
void ctOption::slotColorChangedToBlack()
{
  if (BLACK != currentColor)
  {
    // Set current color
    currentColor = BLACK;

    // Set new pixmap for selectedPiece button
    pbSelectedPiece->setPixmap(pixMaps[currentPiece+8]);

    // Set new piece for DrawBoard widget
    dbDrawingBoard->slotSetPiece(currentPiece+8);
  }

}

/** Returns the number (1--3) of the line edit that currently has the
input focus, 0 otherwise.
@return Number of the active line edit
*/
int ctOption::getActiveLineEdit()
{
  if (leTitle->hasFocus() == true)
    return(1);
  if (mleTask->hasFocus() == true)
    return(2);
  if (mleSolution->hasFocus() == true)
    return(3);

  return(0);
}

/** Is called when the ``Bold Face'' button was selected.
Inserts the TeX command \c {\\bf } for setting bold face into the
line edit that currently has the focus.
*/
void ctOption::slotInsertBoldFaceCommand()
{
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 1:leTitle->insert("{\\bf }");
           leTitle->setCursorPosition(leTitle->cursorPosition() - 1);
           break;
    case 2:mleTask->insert("{\\bf }");
           mleTask->getCursorPosition(&line, &col);
  	   mleTask->setCursorPosition(line, col-1);
           break;
    case 3:mleSolution->insert("{\\bf }");
           mleSolution->getCursorPosition(&line, &col);
	   mleSolution->setCursorPosition(line, col-1);
           break;
  }  
}
 
/** Is called when the ``Italic Face'' button was selected.
Inserts the TeX command \c {\\it } for setting italic face into the
line edit that currently has the focus.
*/
void ctOption::slotInsertItalicFaceCommand()
{
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 1:leTitle->insert("{\\it }");
           leTitle->setCursorPosition(leTitle->cursorPosition() - 1);
           break;
    case 2:mleTask->insert("{\\it }");
           mleTask->getCursorPosition(&line, &col);
	   mleTask->setCursorPosition(line, col-1);
           break;
    case 3:mleSolution->insert("{\\it }");
           mleSolution->getCursorPosition(&line, &col);
	   mleSolution->setCursorPosition(line, col-1);
           break;
  }  
}
 
/** Is called when the ``Insert'' button was pushed.
Inserts the LaTeX command for the selected ``Informator'' symbol into the
line edit that currently has the focus.
*/
void ctOption::slotInsertInformatorCommand()
{
  // Selected Informator symbol
  int index = lbInformator->currentItem();
  QString sSymbol;

  switch (index)
  {
    case  0: sSymbol = "\\mate";
                       break;
    case  1: sSymbol = "\\wbetter";
                       break;
    case  2: sSymbol = "\\bbetter";
                       break;
    case  3: sSymbol = "\\wupperhand";
                       break;
    case  4: sSymbol = "\\bupperhand";
                       break;
    case  5: sSymbol = "\\wdecisive";
                       break;
    case  6: sSymbol = "\\bdecisive";
                       break;
    case  7: sSymbol = "\\equal";
                       break;
    case  8: sSymbol = "\\unclear";
                       break;
    case  9: sSymbol = "\\compensation";
                       break;
    case  10: sSymbol = "\\devadvantage";
                       break;
    case  11: sSymbol = "\\moreroom";
                       break;
    case  12: sSymbol = "\\withattack";
                       break;
    case  13: sSymbol = "\\withinit";
                       break;
    case  14: sSymbol = "\\counterplay";
                       break;
    case  15: sSymbol = "\\zugzwang";
                       break;
    case  16: sSymbol = "\\withidea";
                       break;
    case  17: sSymbol = "\\onlymove";
                       break;
    case  18: sSymbol = "\\betteris";
                       break;
    case  19: sSymbol = "\\file";
                       break;
    case  20: sSymbol = "\\diagonal";
                       break;
    case  21: sSymbol = "\\centre";
                       break;
    case  22: sSymbol = "\\kside";
                       break;
    case  23: sSymbol = "\\qside";
                       break;
    case  24: sSymbol = "\\weakpt";
                       break;
    case  25: sSymbol = "\\ending";
                       break;
    case  26: sSymbol = "\\bishoppair";
                       break;
    case  27: sSymbol = "\\opposbishops";
                       break;
    case  28: sSymbol = "\\samebishops";
                       break;
    case  29: sSymbol = "\\unitedpawns";
                       break;
    case  30: sSymbol = "\\seppawns";
                       break;
    case  31: sSymbol = "\\doublepawns";
                       break;
    case  32: sSymbol = "\\passedpawn";
                       break;
    case  33: sSymbol = "\\morepawns";
                       break;
    case  34: sSymbol = "\\timelimit";
                       break;
    case  35: sSymbol = "\\novelty";
                       break;
    case  36: sSymbol = "\\comment";
                       break;
    case  37: sSymbol = "\\various";
                       break;
    case  38: sSymbol = "\\without";
                       break;
    case  39: sSymbol = "\\with";
                       break;
    case  40: sSymbol = "\\etc";
                       break;
    case  41: sSymbol = "\\see";
                       break;
    case  42: sSymbol = "\\markera";
                       break;
    case  43: sSymbol = "\\markerb";
                       break;
    default: sSymbol = "";
                       break;
  }		       

  // Append a space to the symbol?
  if (cbTrailingSpace->isChecked() == true)
  {
    // Yes
    sSymbol.append("{} ");
  }
  else
  {
    // No
    sSymbol.append(" ");
  }  

  switch (getActiveLineEdit())
  {
    case 1:leTitle->insert(sSymbol);
           break;
    case 2:mleTask->insert(sSymbol);
           break;
    case 3:mleSolution->insert(sSymbol);
           break;
  }  
}
 
/** Is called when the ``Move'' button was selected.
Inserts the LaTeX command for a move into the
line edit that currently has the focus.
*/
void ctOption::slotInsertMoveCommand()
{ 
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 1:leTitle->insert("\\movecomment{}");
           leTitle->setCursorPosition(leTitle->cursorPosition() - 1);
           break;
    case 2:mleTask->insert("\\movecomment{}");
           mleTask->getCursorPosition(&line, &col);
	   mleTask->setCursorPosition(line, col-1);
           break;
    case 3:mleSolution->insert("\\movecomment{}");
           mleSolution->getCursorPosition(&line, &col);
	   mleSolution->setCursorPosition(line, col-1);
           break;
  }  
}

/** Is called when the key combination ``CTRL+L'' was pressed.
Inserts the ``latexhtml'' command into the
line edit that currently has the focus.
*/
void ctOption::slotInsertLaTeXHTMLCommand()
{ 
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 1:leTitle->insert("\\latexhtml{}{}");
           leTitle->setCursorPosition(leTitle->cursorPosition() - 3);
           break;
    case 2:mleTask->insert("\\latexhtml{}{}");
           mleTask->getCursorPosition(&line, &col);
	   mleTask->setCursorPosition(line, col-3);
           break;
    case 3:mleSolution->insert("\\latexhtml{}{}");
           mleSolution->getCursorPosition(&line, &col);
	   mleSolution->setCursorPosition(line, col-3);
           break;
  }  
}

/** Is called when the key combination ``CTRL+K'' was pressed.
Inserts the ``knightmove'' command into the
``task'' line edit if it has the focus.
*/
void ctOption::slotInsertKnightMoveCommand()
{ 
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 2:mleTask->insert("\\printknightmove{}{}");
           mleTask->getCursorPosition(&line, &col);
	   mleTask->setCursorPosition(line, col-3);
           break;
  }  
}

/** Is called when the key combination ``CTRL+A'' was pressed.
Inserts the ``arrow'' command into the
``task'' line edit if it has the focus.
*/
void ctOption::slotInsertArrowCommand()
{ 
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 2:mleTask->insert("\\printarrow{}{}");
           mleTask->getCursorPosition(&line, &col);
	   mleTask->setCursorPosition(line, col-3);
           break;
  }  
}

/** Is called when the key combination ``CTRL+H'' was pressed.
Inserts the ``highlight'' command into the
``task'' line edit if it has the focus.
*/
void ctOption::slotInsertHighlightCommand()
{ 
  // Row and column in the line edit
  int line, col;

  switch (getActiveLineEdit())
  {
    case 2:mleTask->insert("\\highlight[]{}");
           mleTask->getCursorPosition(&line, &col);
	   mleTask->setCursorPosition(line, col-3);
           break;
  }  
}


